import pytest
import requests
from pactman import Pact, Term


def test_regex_passes():
    pact = (
        Pact("Consumer", "Provider", file_write_mode="never")
        .given("spam")
        .with_request("GET", "/", headers={"Spam": Term(r"\w+", "spam")})
        .will_respond_with(200)
    )
    with pact:
        requests.get(pact.uri, headers={"Spam": "ham"})


def test_regex_fails():
    pact = (
        Pact("Consumer", "Provider", file_write_mode="never")
        .given("spam")
        .with_request("GET", "/", headers={"Spam": Term(r"\w+", "spam")})
        .will_respond_with(200)
    )
    with pact:
        with pytest.raises(AssertionError):
            requests.get(pact.uri, headers={"Spam": "!@#$"})


def test_regex_passes_v3():
    pact = (
        Pact("Consumer", "Provider", file_write_mode="never", version="3.0.0")
        .given("spam")
        .with_request("GET", "/", headers={"Spam": Term(r"\w+", "spam")})
        .will_respond_with(200)
    )
    with pact:
        requests.get(pact.uri, headers={"Spam": "ham"})


def test_regex_fails_v3():
    pact = (
        Pact("Consumer", "Provider", file_write_mode="never", version="3.0.0")
        .given("spam")
        .with_request("GET", "/", headers={"Spam": Term(r"\w+", "spam")})
        .will_respond_with(200)
    )
    with pact:
        with pytest.raises(AssertionError):
            requests.get(pact.uri, headers={"Spam": "!@#$"})



import logging

from colorama import Fore, Style

from .paths import format_path

log = logging.getLogger(__name__)


class Result:
    PASS = True
    FAIL = False
    success = PASS

    def start(self, message):
        self.success = self.PASS

    def end(self):
        pass

    def warn(self, message):
        raise NotImplementedError()

    def fail(self, message, path=None):
        raise NotImplementedError()  # pragma: no cover


class LoggedResult(Result):
    def start(self, interaction):
        super().start(interaction)
        log.info(f"Verifying {interaction}")

    def warn(self, message):
        log.warning(" " + message)

    def fail(self, message, path=None):
        self.success = self.FAIL
        log.warning(" " + message)
        return not message


class PytestResult(Result):  # pragma: no cover
    def __init__(self, *, level=logging.WARNING):
        self.records = []
        self.level = level
        self.current_consumer = None

    def start(self, interaction):
        super().start(interaction)
        log = logging.getLogger("pactman")
        log.handlers = [self]
        log.setLevel(logging.DEBUG)
        log.propagate = False
        self.records[:] = []

    def handle(self, record):
        self.records.append(record)

    def fail(self, message, path=None):
        from _pytest.outcomes import Failed

        __tracebackhide__ = True
        self.success = self.FAIL
        if path:
            message += " at " + format_path(path)
        log.error(message)
        raise Failed(message) from None

    def warn(self, message):
        log.warning(message)

    def results_for_terminal(self):
        for record in self.records:
            if record.levelno > logging.WARN:
                yield record.msg, dict(red=True)
            elif record.levelno > logging.INFO:
                yield record.msg, dict(yellow=True)
            else:
                yield record.msg, {}


class CaptureResult(Result):
    def __init__(self, *, level=logging.WARNING):
        self.messages = []
        self.level = level
        self.current_consumer = None

    def start(self, interaction):
        super().start(interaction)
        log = logging.getLogger("pactman")
        log.handlers = [self]
        log.setLevel(logging.DEBUG)
        self.messages[:] = []
        if self.current_consumer != interaction.pact.consumer:
            print(f"{Style.BRIGHT}Consumer: {interaction.pact.consumer}")
            self.current_consumer = interaction.pact.consumer
        print(f'Request: "{interaction.description}" ... ', end="")

    def end(self):
        if self.success:
            print(Fore.GREEN + "PASSED")
        else:
            print(Fore.RED + "FAILED")
        if self.messages:
            print((Fore.RESET + "\n").join(self.messages))

    def warn(self, message):
        log.warning(message)

    def fail(self, message, path=None):
        self.success = self.FAIL
        if path:
            message += " at " + format_path(path)
        log.error(message)
        return not message

    def handle(self, record):
        color = ""
        if record.levelno >= logging.ERROR:
            color = Fore.RED
        elif record.levelno >= logging.WARNING:
            color = Fore.YELLOW
        self.messages.append(" " + color + record.msg)



import pytest
from pactman.verifier.parse_header import Part, parse_header


def test_comma_whitespace_ignored():
    assert list(parse_header("spam, ham")) == list(parse_header("spam,ham"))


@pytest.mark.parametrize(
    ["header", "result"],
    [
        (
            ("audio/*; q=0.2, audio/basic"),
            [Part(["audio/*"], []), Part(["audio/basic"], [("q", "0.2")])],
        ),
        (
            (
                """text/plain; q=0.5, text/html,
         text/x-dvi; q="0.8", text/x-c"""
            ),
            [
                Part(["text/plain"], []),
                Part(["text/html", "text/x-dvi"], [("q", "0.5")]),
                Part(["text/x-c"], [("q", "0.8")]),
            ],
        ),
        (
            ('"xyz\\"zy",W/"r2d2,xxxx","spam"'),
            [Part(['"xyz\\"zy"', 'W/"r2d2,xxxx"', '"spam"'], [])],
        ),
    ],
)
def test_accept_variants(header, result):
    assert list(parse_header(header)) == result



import json
import logging
import sys
from unittest.mock import Mock

import semver
from pactman.test.test_verifier import FakeResponse
from pactman.verifier.result import LoggedResult
from pactman.verifier.verify import ResponseVerifier

logging.basicConfig(level=logging.DEBUG)

result = LoggedResult()
version = sys.argv[1]
verifier = ResponseVerifier
response = FakeResponse

with open(sys.argv[2]) as file:
    case = json.load(file)

    pact = Mock(
        provider="SpamProvider",
        consumer="SpamConsumer",
        version=version,
        semver=semver.VersionInfo.parse(version),
    )
    verifier(pact, case["expected"], result).verify(response(case["actual"]))



import pytest
from pactman import EachLike, Like, Term


def test_default_options():
    assert EachLike(1).ruby_protocol() == {"json_class": "Pact::ArrayLike", "contents": 1, "min": 1}


def test_minimum():
    assert EachLike(1, minimum=2).ruby_protocol() == {
        "json_class": "Pact::ArrayLike",
        "contents": 1,
        "min": 2,
    }


def test_minimum_assertion_error():
    with pytest.raises(AssertionError) as e:
        EachLike(1, minimum=0)
    assert str(e.value) == "Minimum must be greater than or equal to 1"


def test_nested_matchers():
    matcher = EachLike({"username": Term("[a-z]+", "user"), "id": Like(123)})
    assert matcher.ruby_protocol() == {
        "json_class": "Pact::ArrayLike",
        "contents": {
            "username": {
                "json_class": "Pact::Term",
                "data": {
                    "matcher": {"json_class": "Regexp", "s": "[a-z]+", "o": 0},
                    "generate": "user",
                },
            },
            "id": {"json_class": "Pact::SomethingLike", "contents": 123},
        },
        "min": 1,
    }



from pactman import Consumer, EachLike, Provider
from pactman.mock.request import Request
from pactman.mock.response import Response


def test_eachlike():
    pact = (
        Consumer("consumer")
        .has_pact_with(Provider("provider"), version="2.0.0")
        .given("the condition exists")
        .upon_receiving("a request")
        .with_request("POST", "/path", body=EachLike(1))
        .will_respond_with(200, body={"results": EachLike(1)})
    )

    result = pact.construct_pact(pact._interactions[0])
    assert result == {
        "consumer": {"name": "consumer"},
        "provider": {"name": "provider"},
        "interactions": [
            {
                "description": "a request",
                "providerState": "the condition exists",
                "request": dict(
                    method="POST",
                    path="/path",
                    body=[1],
                    matchingRules={"$.body": {"match": "type", "min": 1}},
                ),
                "response": dict(
                    status=200,
                    body={"results": [1]},
                    matchingRules={"$.body.results": {"match": "type", "min": 1}},
                ),
            }
        ],
        "metadata": dict(pactSpecification=dict(version="2.0.0")),
    }


def test_falsey_request_body():
    target = Request("GET", "/path", body=[])
    assert target.json("2.0.0") == {"method": "GET", "path": "/path", "body": []}


def test_falsey_response_body():
    target = Response(200, body=[])
    assert target.json("2.0.0") == {"status": 200, "body": []}



# -*- encoding: utf8 -*-
from unittest.mock import Mock, call, patch

import urllib3
import urllib3.poolmanager
from urllib3.response import HTTPResponse

from pactman.mock.mock_urlopen import MockURLOpenHandler, patcher


def test_patched_urlopen_calls_service_with_request_parameters():
    pact = Mock(port=1234)
    mock_service = Mock(pact=pact, return_value=HTTPResponse())
    try:
        patcher.add_service(mock_service)
        http = urllib3.PoolManager()
        response = http.request("GET", "http://api.test:1234/path")
    finally:
        patcher.remove_service(mock_service)
    assert mock_service.call_args == call("GET", "/path", body=None, headers={})
    assert response is mock_service.return_value


@patch.object(urllib3.connectionpool.HTTPConnectionPool, "urlopen")
def test_patched_urlopen_handles_many_positional_arguments(HTTPConnectionPool_urlopen):
    # urllib3 passes in up to 7 positional arguments to urlopen so we need to ensure
    # our mocked urlopen method handles these
    mock_service = Mock(config=Mock(port=1234), return_value=HTTPResponse())
    try:
        patcher.add_service(mock_service)
        pool = urllib3.poolmanager.pool_classes_by_scheme["http"]("api.test", port=5678)
        pool.urlopen("POST", "/path", "body1", {}, None, True, False)
    finally:
        patcher.remove_service(mock_service)
    expected_call = call("POST", "/path", "body1", {}, None, True, False)
    assert HTTPConnectionPool_urlopen.call_args == expected_call


def test_urlopen_responder_handles_json_body():
    h = MockURLOpenHandler(Mock())

    interaction = dict(response=dict(body={"message": "hello world"}, status=200))
    r = h.respond_for_interaction(interaction)

    assert r.data == b'{"message": "hello world"}'
    assert r.headers["Content-Type"] == "application/json; charset=UTF-8"


def test_urlopen_responder_handles_json_string_body():
    h = MockURLOpenHandler(Mock())

    interaction = dict(response=dict(body="hello world", status=200))
    r = h.respond_for_interaction(interaction)

    assert r.data == b'"hello world"'
    assert r.headers["Content-Type"] == "application/json; charset=UTF-8"


def test_urlopen_responder_handles_json_encoding():
    h = MockURLOpenHandler(Mock())

    interaction = dict(
        response=dict(
            headers={"content-type": "application/json; charset=utf-8"},
            body="héllo world",
            status=200,
        )
    )
    r = h.respond_for_interaction(interaction)

    assert r.data == b'"h\\u00e9llo world"'
    assert r.headers["Content-Type"] == "application/json; charset=utf-8"


def test_urlopen_responder_handles_non_json_body():
    h = MockURLOpenHandler(Mock())

    interaction = dict(
        response=dict(
            headers={"content-type": "text/plain; charset=utf-8"},
            body="héllo world".encode("utf-8"),
            status=200,
        )
    )
    r = h.respond_for_interaction(interaction)

    assert r.data == b"h\xc3\xa9llo world"
    assert r.headers["Content-Type"] == "text/plain; charset=utf-8"



import json
import pathlib
from itertools import chain
from unittest.mock import Mock

import pytest
import requests
import semver
from pactman.verifier.broker_pact import (BrokerPact, BrokerPacts,
                                          PactBrokerConfig, pact_id)
from pactman.verifier.result import Result
from pactman.verifier.verify import (Interaction, RequestVerifier,
                                     ResponseVerifier)
from restnavigator import Navigator

BASE_DIR = pathlib.Path(__file__).absolute().parents[0]


def all_testcases(path, version):
    for entry in path.iterdir():
        if entry.is_file() and entry.suffix == ".json":
            yield str(entry), version
        elif entry.is_dir():
            yield from all_testcases(entry, version)


@pytest.fixture
def fake_interaction():
    return {
        "description": "dummy",
        "request": {"method": "GET", "path": "/users-service/user/alex"},
        "response": {"headers": {}, "status": 200},
    }


@pytest.fixture
def fake_pact(fake_interaction):
    return {
        "provider": {"name": "SpamProvider"},
        "consumer": {"name": "SpamConsumer"},
        "interactions": [fake_interaction],
        "metadata": {"pact-specification": {"version": "2.0.0"}},
    }


@pytest.fixture
def mock_pact():
    def create_mock(version):
        return Mock(
            provider="SpamProvider",
            consumer="SpamConsumer",
            version=version,
            semver=semver.VersionInfo.parse(version),
        )

    return create_mock


@pytest.fixture
def mock_result():
    result = Result()
    result.fail = Mock(return_value=False)
    result.warn = Mock()
    return result


@pytest.fixture
def mock_result_factory(mock_result):
    return Mock(return_value=mock_result)


def test_pact_id():
    assert pact_id(1) == repr(1)


def test_pact_loading(monkeypatch, fake_pact):
    p = BrokerPacts("SpamProvider", PactBrokerConfig("http://broker.example/"))
    mock_pact = Mock(fetch=Mock(return_value=fake_pact))
    mock_provider = Mock(fetch=lambda: None, __getitem__=lambda s, k: [mock_pact])
    monkeypatch.setattr(
        Navigator,
        "hal",
        lambda url, default_curie=None, auth=None, headers=None: {
            "latest-provider-pacts": lambda provider=None: mock_provider
        },
    )

    for e in p.consumers():
        assert str(e) == "Pact between consumer SpamConsumer and provider SpamProvider"
        assert e.provider == "SpamProvider"

    for e in p.all_interactions():
        assert e.description == "dummy"


def test_pact_publish_uses_interaction_result(fake_pact):
    result_factory = Mock(return_value=Mock(success=True))
    mock_result_publisher = Mock()
    broker_pact = {"publish-verification-results": mock_result_publisher}
    p = BrokerPact(fake_pact, result_factory, broker_pact)
    p.publish_result("1.0")
    mock_result_publisher.create.assert_called_once()
    payload = mock_result_publisher.create.call_args[0][0]
    assert payload == {"success": True, "providerApplicationVersion": "1.0"}


@pytest.mark.parametrize(
    "first_success, second_success, combined_success",
    [(False, True, False), (True, False, False), (True, True, True)],
)
def test_pact_publish_aggregates_interaction_results(
    fake_pact, fake_interaction, first_success, second_success, combined_success
):
    result_factory = Mock()
    fake_pact = {**fake_pact, "interactions": [fake_interaction, fake_interaction]}
    result_factory.side_effect = [Mock(success=first_success), Mock(success=second_success)]
    mock_result_publisher = Mock()
    broker_pact = {"publish-verification-results": mock_result_publisher}
    BrokerPact(fake_pact, result_factory, broker_pact).publish_result("1.0")
    payload = mock_result_publisher.create.call_args[0][0]
    assert payload == {"success": combined_success, "providerApplicationVersion": "1.0"}


def test_pact_file_loading(fake_interaction):
    p = BrokerPact.load_file(str(BASE_DIR / "testcases-version-3" / "dummypact.json"))
    assert str(p) == "Pact between consumer SpamConsumer and provider SpamProvider"
    assert p.provider == "SpamProvider"
    for e in p.interactions:
        assert e.description == "dummy"


def test_interaction(mock_pact, mock_result_factory, fake_interaction):
    i = Interaction(mock_pact("2.0.0"), fake_interaction, mock_result_factory)
    assert repr(i) == "<Interaction SpamConsumer:dummy>"


def test_interaction_verify_get(monkeypatch, mock_pact, mock_result_factory, fake_interaction):
    monkeypatch.setattr(requests, "post", Mock())
    monkeypatch.setattr(requests, "get", Mock())
    monkeypatch.setattr(ResponseVerifier, "verify", Mock())
    i = Interaction(mock_pact("2.0.0"), fake_interaction, mock_result_factory)
    i.verify("http://provider.example/", "http://provider.example/pact-setup/")

    requests.post.assert_not_called()
    requests.get.assert_called_with("http://provider.example/users-service/user/alex", headers={})
    i.response.verify.assert_called()


def test_interaction_verify_method_not_supported(
    monkeypatch, mock_pact, mock_result_factory, fake_interaction
):
    monkeypatch.setattr(requests, "get", Mock())
    monkeypatch.setattr(ResponseVerifier, "verify", Mock())
    fake_interaction["request"]["method"] = "FLEBBLE"
    i = Interaction(mock_pact("2.0.0"), fake_interaction, mock_result_factory)
    i.verify("http://provider.example/", "http://provider.example/pact-setup/")
    i.result.fail.assert_called_with("Request method FLEBBLE not implemented in verifier")
    i.response.verify.assert_not_called()


def test_interaction_verify_qs(monkeypatch, mock_pact, mock_result_factory, fake_interaction):
    monkeypatch.setattr(requests, "post", Mock())
    monkeypatch.setattr(requests, "get", Mock())
    monkeypatch.setattr(ResponseVerifier, "verify", Mock())
    fake_interaction["request"]["query"] = "a=b&c=d"
    i = Interaction(mock_pact("2.0.0"), fake_interaction, mock_result_factory)
    i.verify("http://provider.example/", "http://provider.example/pact-setup/")

    requests.get.assert_called_with(
        "http://provider.example/users-service/user/alex", params=dict(a=["b"], c=["d"]), headers={}
    )
    i.response.verify.assert_called()


def test_interaction_verify_post(monkeypatch, mock_pact, mock_result_factory, fake_interaction):
    monkeypatch.setattr(requests, "post", Mock())
    monkeypatch.setattr(requests, "get", Mock())
    monkeypatch.setattr(ResponseVerifier, "verify", Mock())
    fake_interaction["request"]["method"] = "POST"
    fake_interaction["request"]["body"] = "spam"
    i = Interaction(mock_pact("2.0.0"), fake_interaction, mock_result_factory)
    i.verify("http://provider.example/", "http://provider.example/pact-setup/")

    requests.get.assert_not_called()
    requests.post.assert_called_with(
        "http://provider.example/users-service/user/alex", json="spam", headers={}
    )
    i.response.verify.assert_called()


def test_interaction_verify_post_unsupported_content_type(
    monkeypatch, mock_pact, mock_result_factory, fake_interaction
):
    monkeypatch.setattr(requests, "post", Mock())
    monkeypatch.setattr(ResponseVerifier, "verify", Mock())
    fake_interaction["request"]["method"] = "POST"
    fake_interaction["request"]["headers"] = {"Content-Type": "spam/ham"}
    i = Interaction(mock_pact("2.0.0"), fake_interaction, mock_result_factory)
    i.verify("http://provider.example/", "http://provider.example/pact-setup/")
    i.result.fail.assert_called_with("POST content type spam/ham not implemented in verifier")

    requests.post.assert_not_called()


def test_interaction_verify_delete(monkeypatch, mock_pact, mock_result_factory):
    monkeypatch.setattr(requests, "delete", Mock())
    monkeypatch.setattr(ResponseVerifier, "verify", Mock())
    fake_interaction = {
        "description": "dummy",
        "request": {"method": "DELETE", "path": "/diary-notes/diary-note/1"},
        "response": {"headers": {}, "status": 200},
    }
    i = Interaction(mock_pact("2.0.0"), fake_interaction, mock_result_factory)
    i.verify("http://provider.example/", "http://provider.example/pact-setup/")

    requests.delete.assert_called_with(
        "http://provider.example/diary-notes/diary-note/1", headers={}
    )
    i.response.verify.assert_called()


def test_interaction_verify_put(monkeypatch, mock_pact, mock_result_factory, fake_interaction):
    monkeypatch.setattr(requests, "put", Mock())
    monkeypatch.setattr(requests, "get", Mock())
    monkeypatch.setattr(ResponseVerifier, "verify", Mock())
    fake_interaction["request"]["method"] = "PUT"
    fake_interaction["request"]["body"] = "spam"
    i = Interaction(mock_pact("2.0.0"), fake_interaction, mock_result_factory)
    i.verify("http://provider.example/", "http://provider.example/pact-setup/")

    requests.get.assert_not_called()
    requests.put.assert_called_with(
        "http://provider.example/users-service/user/alex", json="spam", headers={}
    )
    i.response.verify.assert_called()


def test_interaction_verify_put_unsupported_content_type(
    monkeypatch, mock_pact, mock_result_factory, fake_interaction
):
    monkeypatch.setattr(requests, "put", Mock())
    monkeypatch.setattr(ResponseVerifier, "verify", Mock())
    fake_interaction["request"]["method"] = "PUT"
    fake_interaction["request"]["headers"] = {"Content-Type": "spam/ham"}
    i = Interaction(mock_pact("2.0.0"), fake_interaction, mock_result_factory)
    i.verify("http://provider.example/", "http://provider.example/pact-setup/")
    i.result.fail.assert_called_with("PUT content type spam/ham not implemented in verifier")

    requests.put.assert_not_called()


def test_interaction_verify_patch(monkeypatch, mock_pact, mock_result_factory, fake_interaction):
    monkeypatch.setattr(requests, "patch", Mock())
    monkeypatch.setattr(requests, "get", Mock())
    monkeypatch.setattr(ResponseVerifier, "verify", Mock())
    fake_interaction["request"]["method"] = "PATCH"
    fake_interaction["request"]["body"] = "spam"
    i = Interaction(mock_pact("2.0.0"), fake_interaction, mock_result_factory)
    i.verify("http://provider.example/", "http://provider.example/pact-setup/")
    requests.get.assert_not_called()
    requests.patch.assert_called_with(
        "http://provider.example/users-service/user/alex", json="spam", headers={}
    )
    i.response.verify.assert_called()


def test_interaction_verify_patch_unsupported_content_type(
    monkeypatch, mock_pact, mock_result_factory, fake_interaction
):
    monkeypatch.setattr(requests, "patch", Mock())
    monkeypatch.setattr(ResponseVerifier, "verify", Mock())
    fake_interaction["request"]["method"] = "PATCH"
    fake_interaction["request"]["headers"] = {"Content-Type": "spam/ham"}
    i = Interaction(mock_pact("2.0.0"), fake_interaction, mock_result_factory)
    i.verify("http://provider.example/", "http://provider.example/pact-setup/")
    i.result.fail.assert_called_with("PATCH content type spam/ham not implemented in verifier")
    requests.patch.assert_not_called()


@pytest.mark.parametrize("method", ["GET", "POST", "DELETE", "PUT", "PATCH"])
def test_interaction_sends_headers(
    monkeypatch, mock_pact, mock_result_factory, fake_interaction, method
):
    headers = {"key1": "value1"}
    requests_method = Mock()
    monkeypatch.setattr(requests, method.lower(), requests_method)
    monkeypatch.setattr(ResponseVerifier, "verify", Mock())
    fake_interaction["request"]["method"] = method
    fake_interaction["request"]["body"] = "body-data"
    fake_interaction["request"]["headers"] = headers
    i = Interaction(mock_pact("2.0.0"), fake_interaction, mock_result_factory)
    i.verify("http://provider.example/", "http://provider.example/pact-setup/")
    request_kwargs = requests_method.call_args[1]
    assert request_kwargs["headers"] == headers


@pytest.mark.parametrize("option, arg", [("providerState", "state"), ("providerStates", "states")])
def test_interaction_verify_with_setup(
    monkeypatch, mock_pact, mock_result_factory, fake_interaction, option, arg
):
    monkeypatch.setattr(requests, "post", Mock(return_value=Mock(status_code=200)))
    monkeypatch.setattr(requests, "get", Mock())
    monkeypatch.setattr(ResponseVerifier, "verify", Mock())
    fake_interaction[option] = "some state"
    i = Interaction(mock_pact("2.0.0"), fake_interaction, mock_result_factory)
    i.verify("http://provider.example/", "http://provider.example/pact-setup/")

    requests.post.assert_called_with(
        "http://provider.example/pact-setup/",
        json={"provider": "SpamProvider", "consumer": "SpamConsumer", arg: "some state"},
    )
    i.response.verify.assert_called()


@pytest.mark.parametrize("option", ["providerState", "providerStates"])
def test_interaction_setup_fails(
    monkeypatch, mock_pact, mock_result_factory, fake_interaction, option
):
    monkeypatch.setattr(requests, "post", Mock(return_value=Mock(status_code=400, text="fail")))
    monkeypatch.setattr(requests, "get", Mock())
    monkeypatch.setattr(ResponseVerifier, "verify", Mock())
    fake_interaction[option] = "some state"
    i = Interaction(mock_pact("2.0.0"), fake_interaction, mock_result_factory)
    i.verify("http://provider.example/", "http://provider.example/pact-setup/")
    i.result.warn.assert_called_with("Invalid provider state 'some state'")


def test_interaction_setup_connection_fails(
    monkeypatch, mock_pact, mock_result_factory, fake_interaction
):
    monkeypatch.setattr(
        requests, "post", Mock(side_effect=requests.exceptions.ConnectionError("barf"))
    )
    monkeypatch.setattr(requests, "get", Mock())
    # monkeypatch.setattr(ResponseVerifier, 'verify', Mock())
    fake_interaction["providerState"] = "some state"
    i = Interaction(mock_pact("2.0.0"), fake_interaction, mock_result_factory)
    i.set_versioned_provider_state("http://provider.example/pact-setup/", "state", "some state")
    i.result.fail.assert_called_once()


def test_response_verifier(fake_interaction, mock_pact):
    fake_interaction["response"]["body"] = dict(a="b", c=1)  # note: no rule for matching of c
    fake_interaction["response"]["matchingRules"] = {"$.body.a": dict(match="type")}
    fake_interaction["response"]["headers"] = {"Content-Type": "json-yeah"}
    r = ResponseVerifier(mock_pact("2.0.0"), fake_interaction["response"], Mock())
    r.verify(
        Mock(
            status_code=200,
            headers={"Content-Type": "json-yeah"},
            json=Mock(return_value=dict(a="b", c="c")),
        )
    )


@pytest.mark.parametrize(
    "interaction",
    [
        {"body": {"key1": 1}},
        {"body": {"key1": 1}, "matchingRules": {"$.body.key1": {"match": "type"}}},
    ],
)
def test_ResponseVerifier_reports_correct_message_when_key_missing(mock_pact, interaction):
    mock_result = Mock()
    r = ResponseVerifier(mock_pact("2.0.0"), interaction, mock_result)
    r.verify(FakeResponse({"body": {}}))
    mock_result.fail.assert_called_with("Response element 'key1' is missing", ["body"])


@pytest.mark.parametrize(
    "interaction",
    [
        {"body": {"key1": 1}},
        {"body": {"key1": 1}, "matchingRules": {"$.body.key1": {"match": "type"}}},
    ],
)
def test_RequestVerifier_reports_correct_message_when_key_missing(mock_pact, interaction):
    mock_result = Mock()
    r = RequestVerifier(mock_pact("2.0.0"), interaction, mock_result)
    r.verify(FakeRequest({"body": {}}))
    mock_result.fail.assert_called_with("Request element 'key1' is missing", ["body"])


def test_bug85_exact_alongside_type_match_fails(mock_pact):
    # odd bug where an exact match inside a container with a typed match rule
    # would not invoke fail()
    mock_result = Mock()
    r = ResponseVerifier(
        mock_pact("2.0.0"),
        {"body": {"id": "spam"}, "matchingRules": {"$.body.token": {"match": "type"}}},
        mock_result,
    )
    r.verify(FakeResponse({"body": {"id": "ham"}}))
    mock_result.fail.assert_called_with(
        "Element mismatch 'ham' is not expected 'spam'", ["body", "id"]
    )


def test_ResponseVerifier_checks_for_missing_headers(mock_pact):
    interaction = {"headers": {"X-Custom-Header": "value1"}}
    mock_result = Mock()
    r = ResponseVerifier(mock_pact("2.0.0"), interaction, mock_result)
    assert r.verify(FakeResponse({"headers": {}})) is False
    mock_result.fail.assert_called_with("Response missing header 'X-Custom-Header'")


def test_RequestVerifier_checks_for_missing_headers(mock_pact):
    interaction = {"headers": {"X-Custom-Header": "value1"}}
    mock_result = Mock()
    r = RequestVerifier(mock_pact("2.0.0"), interaction, mock_result)
    assert r.verify(FakeRequest({"headers": {}})) is False
    mock_result.fail.assert_called_with("Request missing header 'X-Custom-Header'")


def test_provider_state_uses_custom_headers(
    monkeypatch, mock_pact, mock_result_factory, fake_interaction
):
    fake_interaction["request"]["headers"] = dict(request_header="value")
    monkeypatch.setattr(requests, "post", Mock(return_value=Mock(status_code=200)))
    monkeypatch.setattr(requests, "get", Mock())
    monkeypatch.setattr(ResponseVerifier, "verify", Mock())
    fake_interaction["providerStates"] = "some state"

    i = Interaction(mock_pact("2.0.0"), fake_interaction, mock_result_factory)
    i.verify(
        "http://provider.example/",
        "http://provider.example/pact-setup/",
        extra_provider_headers={"some_header": "some_header"},
    )

    requests.post.assert_called_with(
        "http://provider.example/pact-setup/",
        json={"provider": "SpamProvider", "consumer": "SpamConsumer", "states": "some state"},
        headers={"some_header": "some_header"},
    )

    requests.get.assert_called_with(
        "http://provider.example/users-service/user/alex",
        headers={"request_header": "value", "some_header": "some_header"},
    )
    i.response.verify.assert_called()


class FakeResponse:
    status = 200
    body = None
    text = "fake response text"

    def __init__(self, attrs):
        self.__dict__.update(attrs)
        self.status_code = self.status

    def __repr__(self):
        return f"<FakeResponse {self.__dict__}>"

    def json(self):
        return self.body


class FakeRequest:
    method = "GET"
    body = None

    def __init__(self, attrs):
        self.__dict__.update(attrs)

    def __repr__(self):
        return f"<FakeRequest {self.__dict__}>"

    def json(self):
        return self.body


@pytest.mark.parametrize(
    "filename, version, verifier, response",
    chain(
        (
            (f, v, ResponseVerifier, FakeResponse)
            for f, v in chain(
                all_testcases(
                    BASE_DIR / "pact-spec-version-1.1" / "testcases" / "response", "1.1.0"
                ),
                all_testcases(BASE_DIR / "pact-spec-version-2" / "testcases" / "response", "2.0.0"),
                all_testcases(BASE_DIR / "pact-spec-version-3" / "testcases" / "response", "3.0.0"),
                all_testcases(BASE_DIR / "testcases-version-2" / "response", "2.0.0"),
                all_testcases(BASE_DIR / "testcases-version-3" / "response", "3.0.0"),
            )
        ),
        (
            (f, v, RequestVerifier, FakeRequest)
            for f, v in chain(
                all_testcases(
                    BASE_DIR / "pact-spec-version-1.1" / "testcases" / "request", "1.1.0"
                ),
                all_testcases(BASE_DIR / "pact-spec-version-2" / "testcases" / "request", "2.0.0"),
                all_testcases(BASE_DIR / "pact-spec-version-3" / "testcases" / "request", "3.0.0"),
                all_testcases(BASE_DIR / "testcases-version-2" / "request", "2.0.0"),
                all_testcases(BASE_DIR / "testcases-version-3" / "request", "3.0.0"),
            )
        ),
    ),
)
def test_testcases(filename, version, verifier, response, mock_pact, mock_result):
    if filename.endswith(" xml.json"):
        # some of the files don't declare the damned content-type!
        raise pytest.skip("XML content type not supported")
    with open(filename) as file:
        try:
            case = json.load(file)
        except json.JSONDecodeError:
            raise pytest.skip("JSON test case mal-formed")
        if case["expected"].get("headers", {}).get("Content-Type", "") == "application/xml":
            raise pytest.skip("XML content type not supported")
        rv = verifier(mock_pact(version), case["expected"], mock_result)
        result = rv.verify(response(case["actual"]))
        success = not bool(rv.result.fail.call_count)
        assert result == success, "fail() was not called for failure here"
        assert case["match"] == success


@pytest.mark.parametrize(
    "url, used_url, auth",
    [
        ("http://some.example/", "http://some.example/", None),
        ("http://user:pass@some.example/", "http://some.example/", ("user", "pass")),
        ("https://some.example/", "https://some.example/", None),
    ],
)
def test_broker_config_url(monkeypatch, url, used_url, auth):
    monkeypatch.setattr(Navigator, "hal", Mock())
    PactBrokerConfig(url).get_broker_navigator()
    Navigator.hal.assert_called_once_with(used_url, auth=auth, default_curie="pb", headers=None)


def test_broker_config_uses_auth_env(monkeypatch):
    monkeypatch.setenv("PACT_BROKER_AUTH", "user:pass")
    monkeypatch.setattr(Navigator, "hal", Mock())
    PactBrokerConfig("http://some.example/").get_broker_navigator()
    Navigator.hal.assert_called_once_with(
        "http://some.example/", auth=("user", "pass"), default_curie="pb", headers=None
    )


def test_broker_config_uses_token_env(monkeypatch):
    monkeypatch.setenv("PACT_BROKER_TOKEN", "token")
    monkeypatch.setattr(Navigator, "hal", Mock())
    PactBrokerConfig("http://some.example/").get_broker_navigator()
    Navigator.hal.assert_called_once_with(
        "http://some.example/",
        auth=None,
        default_curie="pb",
        headers=dict(Authorization="Bearer token"),
    )



import pytest

from pactman.mock.matchers import (
    generate_ruby_protocol,
    get_generated_values,
    EachLike,
    Like,
    Term,
    get_matching_rules_v2,
    get_matching_rules_v3,
    Equals,
)


def test_generated_value_unknown_type():
    with pytest.raises(ValueError):
        get_generated_values(set())


@pytest.mark.parametrize(
    "input, output",
    [
        (None, None),
        (False, False),
        ("testing", "testing"),
        (123, 123),
        (3.14, 3.14),
        (
            {"id": 123, "info": {"active": False, "user": "admin"}},
            {"id": 123, "info": {"active": False, "user": "admin"}},
        ),
        ([1, 123, "sample"], [1, 123, "sample"]),
        (EachLike({"a": 1}), [{"a": 1}]),
        (EachLike({"a": 1}, minimum=5), [{"a": 1}] * 5),
        (Like(123), 123),
        (Term("[a-f0-9]+", "abc123"), "abc123"),
        (Equals(["a", Like("b")]), ["a", "b"]),
        (
            [EachLike({"username": Term("[a-zA-Z]+", "firstlast"), "id": Like(123)})],
            [[{"username": "firstlast", "id": 123}]],
        ),
    ],
)
def test_generation(input, output):
    assert get_generated_values(input) == output


def test_matching_rules_v2_invald_type():
    with pytest.raises(ValueError):
        assert get_matching_rules_v2(set(), "*")


def test_matching_rules_v3_invald_type():
    with pytest.raises(ValueError):
        assert get_matching_rules_v3(set(), "*")


@pytest.mark.parametrize(
    "input, output",
    [
        (None, None),
        ("testing", "testing"),
        (123, 123),
        (3.14, 3.14),
        (
            {"id": 123, "info": {"active": False, "user": "admin"}},
            {"id": 123, "info": {"active": False, "user": "admin"}},
        ),
        ([1, 123, "sample"], [1, 123, "sample"]),
        (EachLike({"a": 1}), {"json_class": "Pact::ArrayLike", "contents": {"a": 1}, "min": 1}),
        (Like(123), {"json_class": "Pact::SomethingLike", "contents": 123}),
        (
            Term("[a-f0-9]+", "abc123"),
            {
                "json_class": "Pact::Term",
                "data": {
                    "matcher": {"json_class": "Regexp", "s": "[a-f0-9]+", "o": 0},
                    "generate": "abc123",
                },
            },
        ),
        (
            [EachLike({"username": Term("[a-zA-Z]+", "firstlast"), "id": Like(123)})],
            [
                {
                    "contents": {
                        "id": {"contents": 123, "json_class": "Pact::SomethingLike"},
                        "username": {
                            "data": {
                                "generate": "firstlast",
                                "matcher": {"json_class": "Regexp", "o": 0, "s": "[a-zA-Z]+"},
                            },
                            "json_class": "Pact::Term",
                        },
                    },
                    "json_class": "Pact::ArrayLike",
                    "min": 1,
                }
            ],
        ),
    ],
)
def test_from_term(input, output):
    assert generate_ruby_protocol(input) == output


def test_ruby_protocol_unknown_type():
    with pytest.raises(ValueError):
        generate_ruby_protocol(set())



import os
import tempfile
from unittest import TestCase
from unittest.mock import call, patch

import requests
from pactman.mock.consumer import Consumer
from pactman.mock.pact import Pact
from pactman.mock.provider import Provider


class PactTestCase(TestCase):
    def setUp(self):
        self.consumer = Consumer("TestConsumer")
        self.provider = Provider("TestProvider")

    def test_init_defaults(self):
        target = Pact(self.consumer, self.provider)
        self.assertIs(target.consumer, self.consumer)
        self.assertEqual(target.host_name, "localhost")
        self.assertEqual(target.log_dir, os.getcwd())
        self.assertEqual(target.pact_dir, os.getcwd())
        self.assertEqual(target.port, Pact.BASE_PORT_NUMBER)
        self.assertIs(target.provider, self.provider)
        self.assertIs(target.ssl, False)
        self.assertIsNone(target.sslcert)
        self.assertIsNone(target.sslkey)
        self.assertEqual(target.uri, f"http://localhost:{Pact.BASE_PORT_NUMBER}")
        self.assertEqual(target.version, "2.0.0")
        self.assertEqual(len(target._interactions), 0)

    def test_init_custom_mock_service(self):
        with tempfile.TemporaryDirectory() as d:
            target = Pact(
                self.consumer,
                self.provider,
                host_name="192.168.1.1",
                port=8000,
                log_dir="/logs",
                ssl=True,
                sslcert="/ssl.cert",
                sslkey="/ssl.pem",
                pact_dir=d,
                version="3.0.0",
                file_write_mode="merge",
                use_mocking_server=False,
            )

        self.assertIs(target.consumer, self.consumer)
        self.assertEqual(target.host_name, "192.168.1.1")
        self.assertEqual(target.log_dir, "/logs")
        self.assertEqual(target.pact_dir, d)
        self.assertEqual(target.port, 8000)
        self.assertIs(target.provider, self.provider)
        self.assertIs(target.ssl, True)
        self.assertEqual(target.sslcert, "/ssl.cert")
        self.assertEqual(target.sslkey, "/ssl.pem")
        self.assertEqual(target.uri, "https://192.168.1.1:8000")
        self.assertEqual(target.version, "3.0.0")
        self.assertEqual(target.file_write_mode, "merge")
        self.assertEqual(len(target._interactions), 0)
        self.assertIs(target.use_mocking_server, False)

    def test_definition_sparse(self):
        target = Pact(self.consumer, self.provider)
        (
            target.given("I am creating a new pact using the Pact class")
            .upon_receiving("a specific request to the server")
            .with_request("GET", "/path")
            .will_respond_with(200, body="success")
        )

        self.assertEqual(len(target._interactions), 1)

        self.assertEqual(
            target._interactions[0]["providerState"],
            "I am creating a new pact using the Pact class",
        )

        self.assertEqual(target._interactions[0]["description"], "a specific request to the server")

        self.assertEqual(target._interactions[0]["request"], {"path": "/path", "method": "GET"})
        self.assertEqual(target._interactions[0]["response"], {"status": 200, "body": "success"})

    def test_definition_all_options(self):
        target = Pact(self.consumer, self.provider, version="2.0.0")
        (
            target.given("I am creating a new pact using the Pact class")
            .upon_receiving("a specific request to the server")
            .with_request(
                "GET",
                "/path",
                body={"key": "value"},
                headers={"Accept": "application/json"},
                query="search=test",
            )
            .will_respond_with(200, body="success", headers={"Content-Type": "application/json"})
        )

        self.assertEqual(
            target._interactions[0]["providerState"],
            "I am creating a new pact using the Pact class",
        )

        self.assertEqual(target._interactions[0]["description"], "a specific request to the server")

        self.assertEqual(
            target._interactions[0]["request"],
            {
                "path": "/path",
                "method": "GET",
                "body": {"key": "value"},
                "headers": {"Accept": "application/json"},
                "query": "search=test",
            },
        )
        self.assertEqual(
            target._interactions[0]["response"],
            {"status": 200, "body": "success", "headers": {"Content-Type": "application/json"}},
        )

    def test_definition_all_options_v3(self):
        target = Pact(self.consumer, self.provider, version="3.0.0")
        (
            target.given([{"name": "I am creating a new pact using the Pact class", "params": {}}])
            .upon_receiving("a specific request to the server")
            .with_request(
                "GET",
                "/path",
                body={"key": "value"},
                headers={"Accept": "application/json"},
                query={"search": ["test"]},
            )
            .will_respond_with(200, body="success", headers={"Content-Type": "application/json"})
        )

        self.assertEqual(
            target._interactions[0]["providerStates"],
            [{"name": "I am creating a new pact using the Pact class", "params": {}}],
        )

        self.assertEqual(target._interactions[0]["description"], "a specific request to the server")

        self.assertEqual(
            target._interactions[0]["request"],
            {
                "path": "/path",
                "method": "GET",
                "body": {"key": "value"},
                "headers": {"Accept": "application/json"},
                "query": {"search": ["test"]},
            },
        )
        self.assertEqual(
            target._interactions[0]["response"],
            {"status": 200, "body": "success", "headers": {"Content-Type": "application/json"}},
        )

    def test_definition_v3_requires_new_providerStates(self):
        target = Pact(self.consumer, self.provider, version="3.0.0")
        target.given("I am creating a new pact using the Pact class")
        self.assertEqual(
            target._interactions[0]["providerStates"],
            [{"name": "I am creating a new pact using the Pact class", "params": {}}],
        )

    def test_definition_multiple_interactions(self):
        target = Pact(self.consumer, self.provider)
        (
            target.given("I am creating a new pact using the Pact class")
            .upon_receiving("a specific request to the server")
            .with_request("GET", "/foo")
            .will_respond_with(200, body="success")
            .given("I am creating another new pact using the Pact class")
            .upon_receiving("a different request to the server")
            .with_request("GET", "/bar")
            .will_respond_with(200, body="success")
        )

        self.assertEqual(len(target._interactions), 2)

        self.assertEqual(
            target._interactions[1]["providerState"],
            "I am creating a new pact using the Pact class",
        )
        self.assertEqual(
            target._interactions[0]["providerState"],
            "I am creating another new pact using the Pact class",
        )

        self.assertEqual(target._interactions[1]["description"], "a specific request to the server")
        self.assertEqual(
            target._interactions[0]["description"], "a different request to the server"
        )

        self.assertEqual(target._interactions[1]["request"], {"path": "/foo", "method": "GET"})
        self.assertEqual(target._interactions[0]["request"], {"path": "/bar", "method": "GET"})

        self.assertEqual(target._interactions[1]["response"], {"status": 200, "body": "success"})
        self.assertEqual(target._interactions[0]["response"], {"status": 200, "body": "success"})


class PactSetupTestCase(PactTestCase):
    def setUp(self):
        super(PactSetupTestCase, self).setUp()
        self.addCleanup(patch.stopall)
        self.target = Pact(self.consumer, self.provider)
        (
            self.target.given("I am creating a new pact using the Pact class")
            .upon_receiving("a specific request to the server")
            .with_request("GET", "/path")
            .will_respond_with(200, body="success")
        )

        self.delete_call = call(
            "delete", "http://localhost:1234/interactions", headers={"X-Pact-Mock-Service": "true"}
        )

        self.put_interactions_call = call(
            "put",
            "http://localhost:1234/interactions",
            data=None,
            headers={"X-Pact-Mock-Service": "true"},
            json={
                "interactions": [
                    {
                        "response": {"status": 200, "body": "success"},
                        "request": {"path": "/path", "method": "GET"},
                        "description": "a specific request to the server",
                        "providerState": "I am creating a new pact using the Pact class",
                    }
                ]
            },
        )


class PactContextManagerTestCase(PactTestCase):
    def setUp(self):
        super(PactContextManagerTestCase, self).setUp()
        self.addCleanup(patch.stopall)
        self.mock_setup = patch.object(Pact, "setup", autospec=True).start()

        self.mock_verify = patch.object(Pact, "verify", autospec=True).start()

    def test_successful(self):
        pact = Pact(self.consumer, self.provider)
        with pact:
            pass

        self.mock_setup.assert_called_once_with(pact)
        self.mock_verify.assert_called_once_with(pact)

    def test_context_raises_error(self):
        pact = Pact(self.consumer, self.provider)
        with self.assertRaises(RuntimeError):
            with pact:
                raise RuntimeError

        self.mock_setup.assert_called_once_with(pact)
        self.assertFalse(self.mock_verify.called)


def test_multiple_pacts_dont_break_during_teardown():
    # ensure teardown is only done on when all pacts __exit__
    pact = Pact(Consumer("Consumer"), Provider("Provider"))
    p1 = (
        pact.given("given")
        .upon_receiving("when")
        .with_request("GET", "/path")
        .will_respond_with(201)
    )
    p2 = (
        pact.given("given2")
        .upon_receiving("when2")
        .with_request("GET", "/path2")
        .will_respond_with(201)
    )
    with p1, p2:
        requests.get(p1.uri + "/path")



from pactman.verifier.broker_pact import PactBrokerConfig

# NOTE: this code is a WIP, it's not used yet


class Publisher:
    def __init__(self, broker: PactBrokerConfig):
        self.broker = broker
        self.nav = self.broker.get_broker_navigator()

    def publish_pact(self, pact, version, tags):
        # there's no direct link from the broker root to a pacticipant, so we go via pb:latest-version
        tagger = self.nav["latest-version"](pacticipant=pact["consumer"])["pacticipant"][
            "version-tag"
        ]
        for tag in tags:
            tagger(tag=tag, version=version).upsert({})

        self.nav["publish-pact"](
            consumer=pact["consumer"], provider=pact["provider"], version=version
        )



import collections
from unittest.mock import Mock

import pytest
from pactman.verifier.matching_rule import (
    InvalidMatcher,
    MatchDecimal,
    MatchEquality,
    Matcher,
    MatchInclude,
    MatchInteger,
    MatchNull,
    MatchNumber,
    MatchRegex,
    MatchType,
    RuleFailed,
    fold_type,
    log,
    rule_matchers_v2,
    rule_matchers_v3,
    split_path,
    weight_path,
)


def test_stringify():
    r = MatchType("$", {"match": "type"})
    assert str(r) == "Rule match by {'match': 'type'} at $"
    assert repr(r) == "<MatchType path='$' rule={'match': 'type'}>"


def test_invalid_match_type(monkeypatch):
    monkeypatch.setattr(log, "warning", Mock())
    assert isinstance(Matcher.get_matcher("$", {"match": "spam"}), InvalidMatcher)
    log.warning.assert_called_once()


@pytest.mark.parametrize(
    "path, weight, spam_weight",
    [
        ("$.body", 4, 4),
        ("$.body.item1.level[0].id", 0, 0),
        ("$.body.item1.level[1].id", 64, 0),
        ("$.body.item1.level[*].id", 32, 0),
        ("$.body.*.level[*].id", 16, 0),
        ("$.body.*.*[*].id", 8, 8),
    ],
)
def test_weightings(path, weight, spam_weight):
    rule = Matcher(path, {"match": "type"})
    assert rule.weight(["$", "body", "item1", "level", 1, "id"]).weight == weight
    assert rule.weight(["$", "body", "item2", "spam", 1, "id"]).weight == spam_weight


@pytest.mark.parametrize(
    "path, result",
    [
        ("", []),
        ("$", ["$"]),
        ("$.body", ["$", "body"]),
        ("$.body.item1", ["$", "body", "item1"]),
        ("$.body.item2", ["$", "body", "item2"]),
        ("$.header.item1", ["$", "header", "item1"]),
        ("$.body.item1.level", ["$", "body", "item1", "level"]),
        ("$.body.item1.level[1]", ["$", "body", "item1", "level", 1]),
        ("$.body.item1.level[1].id", ["$", "body", "item1", "level", 1, "id"]),
        ("$.body.item1.level[1].name", ["$", "body", "item1", "level", 1, "name"]),
        ("$.body.item1.level[2]", ["$", "body", "item1", "level", 2]),
        ("$.body.item1.level[2].id", ["$", "body", "item1", "level", 2, "id"]),
        ("$.body.item1.level[*].id", ["$", "body", "item1", "level", "*", "id"]),
        ("$.body.*.level[*].id", ["$", "body", "*", "level", "*", "id"]),
    ],
)
def test_split_path(path, result):
    assert list(split_path(path)) == result


@pytest.mark.parametrize(
    "spec, test, result",
    [
        (["a"], ["b"], 0),
        (["*"], ["a"], 1),
        (["a", "*"], ["a", "b"], 2),
        (["a", "b"], ["a", "b"], 4),
        (["a", "b", "c"], ["a", "b"], 0),
        (["a", "b"], ["a", "b", "c"], 4),
    ],
)
def test_weight_path(spec, test, result):
    assert weight_path(spec, test) == result


@pytest.mark.parametrize("data, spec", [(1, 1), (1, 1.0), (1.0, 1.0), (1.0, 1.0)])
def test_numbers(data, spec):
    MatchType("$", dict(match="type")).apply(data, spec, ["a"])


def test_regex():
    MatchRegex("$", dict(match="regex", regex=r"\w+")).apply("spam", None, ["a"])


def test_regex_fail():
    with pytest.raises(RuleFailed):
        MatchRegex("$", dict(match="regex", regex=r"\W+")).apply("spam", None, ["a"])


def test_integer():
    MatchInteger("$", dict(match="integer")).apply(1, None, ["a"])


def test_integer_fail():
    with pytest.raises(RuleFailed):
        MatchInteger("$", dict(match="integer")).apply(1.0, None, ["a"])


def test_decimal():
    MatchDecimal("$", dict(match="decimal")).apply(1.0, None, ["a"])


def test_decimal_fail():
    with pytest.raises(RuleFailed):
        MatchDecimal("$", dict(match="decimal")).apply(1, None, ["a"])


@pytest.mark.parametrize("value", [1, 1.0])
def test_number(value):
    MatchNumber("$", dict(match="number")).apply(value, None, ["a"])


def test_number_fail():
    with pytest.raises(RuleFailed):
        MatchNumber("$", dict(match="number")).apply("spam", None, ["a"])


def test_equality():
    MatchEquality("$", dict(match="equality")).apply("spam", "spam", ["a"])


def test_equality_fail():
    with pytest.raises(RuleFailed):
        MatchEquality("$", dict(match="equality")).apply("ham", "spam", ["a"])


def test_include():
    MatchInclude("$", dict(match="include", value="spam")).apply("spammer", None, ["a"])


def test_include_fail():
    with pytest.raises(RuleFailed):
        MatchInclude("$", dict(match="include", value="spam")).apply("ham", None, ["a"])


def test_null():
    MatchNull("$", dict(match="null")).apply(None, None, ["a"])


def test_null_fail():
    with pytest.raises(RuleFailed):
        MatchNull("$", dict(match="null")).apply("ham", None, ["spam"])


def test_min():
    MatchType("$", dict(match="type", min=1)).apply(["spam"], ["a"], [])


def test_min_not_met():
    with pytest.raises(RuleFailed):
        MatchType("$", dict(match="type", min=2)).apply(["spam"], ["a", "b"], ["spam"])


def test_min_ignored():
    MatchType("$", dict(match="type", min=1)).apply(0, 0, [])


def test_max():
    MatchType("$", dict(match="type", max=1)).apply(["spam"], ["a"], [])


def test_max_not_met():
    with pytest.raises(RuleFailed):
        MatchType("$", dict(match="type", max=2)).apply([1, 2, 3], [1, 2], ["spam"])


def test_max_ignored():
    MatchType("$", dict(match="type", max=1)).apply(0, 0, [])


@pytest.mark.parametrize(
    "source, result", [({}, dict), ([], list), (collections.OrderedDict(), dict)]
)
def test_fold_type(source, result):
    assert fold_type(source) == result


def test_build_matching_rules_handles_rule_with_unknown_type_v2(monkeypatch):
    monkeypatch.setattr(log, "warning", Mock())
    rules = rule_matchers_v2({"$.body": {"match": "SPAM"}, "$.body[*].*": {"match": "type"}})
    assert 2 == len(rules["body"])
    log.warning.assert_called_once()


def test_build_matching_rules_handles_rule_with_unknown_type_v3(monkeypatch):
    monkeypatch.setattr(log, "warning", Mock())
    rules = rule_matchers_v3(
        {
            "body": {
                "$": {"matchers": [{"match": "SPAM"}]},
                "$[*].*": {"matchers": [{"match": "type"}]},
            }
        }
    )
    assert 2 == len(rules["body"])
    log.warning.assert_called_once()



import pytest
from pactman import Like, SomethingLike
from pactman.mock.matchers import Matcher, Term


def test_is_something_like():
    assert SomethingLike is Like


def test_valid_types():
    types = [None, list(), dict(), 1, 1.0, "string", "unicode", Matcher()]

    for t in types:
        SomethingLike(t)


def test_invalid_types():
    with pytest.raises(AssertionError) as e:
        SomethingLike(set())

    assert "matcher must be one of " in str(e.value)


def test_basic_type():
    assert SomethingLike(123).ruby_protocol() == {
        "json_class": "Pact::SomethingLike",
        "contents": 123,
    }


def test_complex_type():
    assert SomethingLike({"name": Term(".+", "admin")}).ruby_protocol() == {
        "json_class": "Pact::SomethingLike",
        "contents": {
            "name": {
                "json_class": "Pact::Term",
                "data": {
                    "matcher": {"json_class": "Regexp", "s": ".+", "o": 0},
                    "generate": "admin",
                },
            }
        },
    }



from pactman import Term
from pactman.mock.request import Request


def test_matcher_in_path_gets_converted():
    target = Request("GET", "/", headers={"Spam": Term(r"\w+", "spam")})
    assert target.json("3.0.0") == {
        "method": "GET",
        "path": "/",
        "headers": {"Spam": "spam"},
        "matchingRules": {"header": {"Spam": {"matchers": [{"match": "regex", "regex": "\\w+"}]}}},
    }



from unittest import TestCase
from unittest.mock import Mock

from pactman.mock.consumer import Consumer
from pactman.mock.pact import Pact
from pactman.mock.provider import Provider


class ConsumerTestCase(TestCase):
    def setUp(self):
        self.mock_service = Mock(Pact)
        self.provider = Mock(Provider)
        self.consumer = Consumer("TestConsumer", service_cls=self.mock_service)

    def test_init(self):
        result = Consumer("TestConsumer")
        self.assertIsInstance(result, Consumer)
        self.assertEqual(result.name, "TestConsumer")
        self.assertIs(result.service_cls, Pact)

    def test_has_pact_with(self):
        result = self.consumer.has_pact_with(self.provider)
        self.assertIs(result, self.mock_service.return_value)
        self.mock_service.assert_called_once_with(
            consumer=self.consumer,
            provider=self.provider,
            host_name="localhost",
            port=None,
            file_write_mode="overwrite",
            log_dir=None,
            ssl=False,
            sslcert=None,
            sslkey=None,
            pact_dir=None,
            version="2.0.0",
            use_mocking_server=False,
        )

    def test_has_pact_with_customer_all_options(self):
        result = self.consumer.has_pact_with(
            self.provider,
            host_name="example.com",
            port=1111,
            log_dir="/logs",
            ssl=True,
            sslcert="/ssl.cert",
            sslkey="ssl.pem",
            pact_dir="/pacts",
            version="3.0.0",
            use_mocking_server=False,
        )

        self.assertIs(result, self.mock_service.return_value)
        self.mock_service.assert_called_once_with(
            consumer=self.consumer,
            provider=self.provider,
            host_name="example.com",
            port=1111,
            file_write_mode="overwrite",
            log_dir="/logs",
            ssl=True,
            sslcert="/ssl.cert",
            sslkey="ssl.pem",
            pact_dir="/pacts",
            version="3.0.0",
            use_mocking_server=False,
        )

    def test_has_pact_with_not_a_provider(self):
        with self.assertRaises(ValueError):
            self.consumer.has_pact_with(None)



import pytest
from pactman import Consumer, Like, Provider, Term
from pactman.mock.request import Request


def test_v2():
    pact = Consumer("consumer").has_pact_with(Provider("provider"), version="2.0.0")

    pact.given("the condition exists").upon_receiving("a request").with_request(
        "GET", "/path", query="fields=first,second"
    ).will_respond_with(200, body="ok")

    result = pact.construct_pact(pact._interactions[0])
    assert result == {
        "consumer": {"name": "consumer"},
        "provider": {"name": "provider"},
        "interactions": [
            {
                "description": "a request",
                "providerState": "the condition exists",
                "request": dict(method="GET", path="/path", query="fields=first,second"),
                "response": dict(status=200, body="ok"),
            }
        ],
        "metadata": dict(pactSpecification=dict(version="2.0.0")),
    }


@pytest.mark.parametrize("query_field", ["first,second", ["first,second"]])
def test_v3(query_field):
    pact = Consumer("consumer").has_pact_with(Provider("provider"), version="3.0.0")
    pact.given([{"name": "the condition exists", "params": {}}]).upon_receiving(
        "a request"
    ).with_request("GET", "/path", query=dict(fields=query_field)).will_respond_with(200, body="ok")

    result = pact.construct_pact(pact._interactions[0])
    assert result == {
        "consumer": {"name": "consumer"},
        "provider": {"name": "provider"},
        "interactions": [
            {
                "description": "a request",
                "providerStates": [{"name": "the condition exists", "params": {}}],
                "request": dict(method="GET", path="/path", query=dict(fields=["first,second"])),
                "response": dict(status=200, body="ok"),
            }
        ],
        "metadata": dict(pactSpecification=dict(version="3.0.0")),
    }


def test_like_v2():
    pact = Consumer("consumer").has_pact_with(Provider("provider"), version="2.0.0")

    pact.given("the condition exists").upon_receiving("a request").with_request(
        "GET", "/path", query=Like("fields=first,second")
    ).will_respond_with(200, body="ok")

    result = pact.construct_pact(pact._interactions[0])
    assert result == {
        "consumer": {"name": "consumer"},
        "provider": {"name": "provider"},
        "interactions": [
            {
                "description": "a request",
                "providerState": "the condition exists",
                "request": dict(
                    method="GET",
                    path="/path",
                    query="fields=first,second",
                    matchingRules={"$.query": {"match": "type"}},
                ),
                "response": dict(status=200, body="ok"),
            }
        ],
        "metadata": dict(pactSpecification=dict(version="2.0.0")),
    }


def test_like_v3():
    pact = (
        Consumer("consumer")
        .has_pact_with(Provider("provider"), version="3.0.0")
        .given("the condition exists")
        .upon_receiving("a request")
        .with_request("GET", "/path", query=dict(fields=Like(["first,second"])))
        .will_respond_with(200, body="ok")
    )

    result = pact.construct_pact(pact._interactions[0])
    assert result == {
        "consumer": {"name": "consumer"},
        "provider": {"name": "provider"},
        "interactions": [
            {
                "description": "a request",
                "providerStates": [{"name": "the condition exists", "params": {}}],
                "request": dict(
                    method="GET",
                    path="/path",
                    query=dict(fields=["first,second"]),
                    matchingRules={"query": {"fields": {"matchers": [{"match": "type"}]}}},
                ),
                "response": dict(status=200, body="ok"),
            }
        ],
        "metadata": dict(pactSpecification=dict(version="3.0.0")),
    }


def test_broader_like_v3():
    pact = (
        Consumer("consumer")
        .has_pact_with(Provider("provider"), version="3.0.0")
        .given("the condition exists")
        .upon_receiving("a request")
        .with_request("GET", "/path", query=Like(dict(fields=["first,second"])))
        .will_respond_with(200, body="ok")
    )

    result = pact.construct_pact(pact._interactions[0])
    assert result == {
        "consumer": {"name": "consumer"},
        "provider": {"name": "provider"},
        "interactions": [
            {
                "description": "a request",
                "providerStates": [{"name": "the condition exists", "params": {}}],
                "request": dict(
                    method="GET",
                    path="/path",
                    query=dict(fields=["first,second"]),
                    matchingRules={"query": {"*": {"matchers": [{"match": "type"}]}}},
                ),
                "response": dict(status=200, body="ok"),
            }
        ],
        "metadata": dict(pactSpecification=dict(version="3.0.0")),
    }


def test_matcher_in_query():
    target = Request("GET", "/test-path", query={"q": [Like("spam")], "l": [Term(r"\d+", "10")]})
    assert target.json("3.0.0") == {
        "method": "GET",
        "path": "/test-path",
        "query": {"q": ["spam"], "l": ["10"]},
        "matchingRules": {
            "query": {
                "q": {"matchers": [{"match": "type"}]},
                "l": {"matchers": [{"match": "regex", "regex": r"\d+"}]},
            }
        },
    }



from pactman import Term
from pactman.mock.request import Request


def test_matcher_in_path_gets_converted():
    target = Request("GET", Term(r"\/.+", "/test-path"))
    assert target.json("2.0.0") == {
        "method": "GET",
        "path": "/test-path",
        "matchingRules": {"$.path": {"regex": r"\/.+"}},
    }



import logging
import queue
import traceback
from http.server import BaseHTTPRequestHandler, HTTPServer
from multiprocessing import Process, Queue

from .pact_request_handler import PactRequestHandler

_providers = {}


log = logging.getLogger(__name__)


def getMockServer(pact):
    if pact.provider.name not in _providers:
        _providers[pact.provider.name] = Server(pact)
    return _providers[pact.provider.name]


class Server:
    def __init__(self, pact):
        self.pact = pact
        self.interactions = Queue()
        self.results = Queue()
        self.process = Process(target=run_server, args=(pact, self.interactions, self.results))
        self.process.start()

    def setup(self, interactions):
        for interaction in interactions:
            self.interactions.put_nowait(interaction)

    def verify(self):
        while not self.results.empty():
            result = self.results.get()
            if result["status"] == "error":
                raise MockServer.Error(result["reason"])
            if result["status"] == "failed":
                raise AssertionError(result["reason"])

    def terminate(self):
        self.process.terminate()


def run_server(pact, interactions, results):
    httpd = MockServer(pact, interactions, results)
    httpd.serve_forever()


class MockServer(HTTPServer):
    def __init__(self, pact, interactions, results):
        self.pact = pact
        self.incoming_interactions = interactions
        self.outgoing_results = results
        server_address = ("", pact.port)
        super().__init__(server_address, MockHTTPRequestHandler)
        self.interactions = []
        self.log = logging.getLogger(__name__ + "." + pact.provider.name)
        self.log.addHandler(logging.FileHandler(f"{pact.log_dir}/{pact.provider.name}.log"))
        self.log.setLevel(logging.DEBUG)
        self.log.propagate = False

    class Error(Exception):
        pass


class MockHTTPRequestHandler(BaseHTTPRequestHandler, PactRequestHandler):
    def __init__(self, request, client_address, server):
        self.response_status_code = None
        self.response_headers = {}
        self.response_body = None
        PactRequestHandler.__init__(self, server.pact)
        BaseHTTPRequestHandler.__init__(self, request, client_address, server)

    def error_result(self, message, content="", status="error", status_code=500):
        self.server.outgoing_results.put({"status": status, "reason": message})
        self.response_status_code = status_code
        self.response_headers = {"Content-Type": "text/plain; charset=utf-8"}
        self.response_body = (content or message).encode("utf8")

    def run_request(self, method):
        try:
            self.body = None
            for header in self.headers:
                if header.lower() == "content-length":
                    self.body = self.rfile.read(int(self.headers[header]))
            self.validate_request(method)
        except AssertionError as e:
            self.error_result(str(e))
        except Exception as e:
            self.error_result(f"Internal Error: {e}", traceback.format_exc())
        self.send_response(self.response_status_code)
        for header in self.response_headers:
            self.send_header(header, self.response_headers[header])
        self.end_headers()
        if self.response_body:
            self.wfile.write(self.response_body)

    def get_interaction(self, path):
        try:
            interaction = self.server.incoming_interactions.get(False)
        except queue.Empty:
            raise AssertionError(
                f"Request at {path} received but no interaction registered"
            ) from None
        return interaction

    def handle_success(self, interaction):
        self.server.outgoing_results.put({"status": "success"})

    def handle_failure(self, reason):
        self.error_result(reason, status="failed", status_code=418)

    def respond_for_interaction(self, interaction):
        self.response_status_code = interaction["response"]["status"]
        if "headers" in interaction["response"]:
            self.response_headers.update(interaction["response"]["headers"])
        if "body" in interaction["response"]:
            self.response_body = self.handle_response_encoding(
                interaction["response"], self.response_headers
            )

    def do_DELETE(self):
        self.run_request("DELETE")

    def do_GET(self):
        self.run_request("GET")

    def do_HEAD(self):
        self.run_request("HEAD")

    def do_POST(self):
        self.run_request("POST")

    def do_PUT(self):
        self.run_request("PUT")

    def do_PATCH(self):
        self.run_request("PATCH")

    def log_message(self, format, *args):
        self.server.log.info("MockServer %s\n" % format % args)



"""Implement verification of pacts as per specification version 2:

https://github.com/pact-foundation/pact-specification/tree/version-2
"""
import json
import os
import urllib.parse

import semver
from restnavigator import Navigator

from .result import LoggedResult
from .verify import Interaction


def pact_id(param):
    return repr(param)


class PactBrokerConfig:
    def __init__(self, url=None, token=None, tags=None):
        url = url or os.environ.get("PACT_BROKER_URL")
        if not url:
            raise ValueError("pact broker URL must be specified")

        # pull the hostname and optionally any basic auth from the broker URL
        # (backwards compat to once upon a time when the broker config URL was specified with a path)
        url_parts = urllib.parse.urlparse(url)
        host = netloc = url_parts.netloc
        self.auth = None
        if "@" in netloc:
            url_auth, host = netloc.split("@")
            self.auth = tuple(url_auth.split(":"))
        self.url = f"{url_parts.scheme}://{host}/"

        if not self.auth:
            auth = os.environ.get("PACT_BROKER_AUTH")
            if auth:
                self.auth = tuple(auth.split(":"))

        token = token or os.environ.get("PACT_BROKER_TOKEN")
        self.headers = None
        if token:
            self.headers = {"Authorization": f"Bearer {token}"}

        self.tags = tags

    def get_broker_navigator(self):
        return Navigator.hal(self.url, default_curie="pb", auth=self.auth, headers=self.headers)

    def get_pacts_for_provider(self, provider):
        if self.tags:
            yield from self.get_tagged_pacts(provider)
        else:
            yield from self.get_all_pacts(provider)

    def get_all_pacts(self, provider):
        nav = self.get_broker_navigator()
        try:
            broker_provider = nav["latest-provider-pacts"](provider=provider)
        except Exception as e:
            raise ValueError(f"error fetching pacts from {self.url} for {provider}: {e}")
        broker_provider.fetch()
        for broker_pact in broker_provider["pacts"]:
            yield broker_pact, broker_pact.fetch()

    def get_tagged_pacts(self, provider):
        nav = self.get_broker_navigator()
        # fetch a set for each tag, just make sure we don't verify the same pact more than once
        seen = set()
        for tag in self.tags:
            try:
                broker_provider = nav["latest-provider-pacts-with-tag"](provider=provider, tag=tag)
            except Exception as e:
                raise ValueError(f"error fetching pacts from {self.url} for {provider}: {e}")
            broker_provider.fetch()
            for broker_pact in broker_provider["pacts"]:
                content = broker_pact.fetch()
                # don't re-run the same pact content
                h = str(content)
                if h in seen:
                    continue
                seen.add(h)
                yield broker_pact, content


class BrokerPacts:
    def __init__(self, provider_name, pact_broker=None, result_factory=LoggedResult):
        self.provider_name = provider_name
        self.pact_broker = pact_broker or PactBrokerConfig()
        self.result_factory = result_factory

    def consumers(self):
        for broker_pact, pact_contents in self.pact_broker.get_pacts_for_provider(
            self.provider_name
        ):
            yield BrokerPact(pact_contents, self.result_factory, broker_pact)

    def all_interactions(self):
        for pact in self.consumers():
            yield from pact.interactions

    def __iter__(self):
        return self.all_interactions()


class BrokerPact:
    def __init__(self, pact, result_factory, broker_pact=None):
        self.pact = pact
        self.provider = pact["provider"]["name"]
        self.consumer = pact["consumer"]["name"]
        self.metadata = pact["metadata"]
        if "pactSpecification" in self.metadata:
            # the Ruby implementation generates non-compliant metadata, handle that :-(
            self.version = self.metadata["pactSpecification"]["version"]
        else:
            self.version = self.metadata["pact-specification"]["version"]
        self.semver = semver.VersionInfo.parse(self.version)
        self.interactions = [
            Interaction(self, interaction, result_factory) for interaction in pact["interactions"]
        ]
        self.broker_pact = broker_pact

    def __repr__(self):
        return f"<Pact consumer={self.consumer} provider={self.provider}>"

    def __str__(self):
        return f"Pact between consumer {self.consumer} and provider {self.provider}"

    @property
    def success(self):
        return all(interaction.result.success for interaction in self.interactions)

    def publish_result(self, version):
        if self.broker_pact is None:
            return
        self.broker_pact["publish-verification-results"].create(
            dict(success=self.success, providerApplicationVersion=version)
        )

    @classmethod
    def load_file(cls, filename, result_factory=LoggedResult):
        with open(filename) as file:
            return cls(json.load(file), result_factory)



import requests

from pactman import Pact


def test_params_url_coded():
    pact = (
        Pact("Consumer", "Provider", file_write_mode="never")
        .given("everything is ideal")
        .upon_receiving("a request")
        .with_request(
            method="POST",
            path="/",
            headers={"Content-Type": "application/x-www-form-urlencoded"},
            query={
                "client_id": ["test1"],
                "secret": ["test1-secret-key"],
                "scope": ["openid", "profile", "phone", "offline_access"],
            },
        )
        .will_respond_with(200, body="some data")
    )
    with pact:
        requests.post(
            pact.uri,
            params=dict(
                client_id="test1",
                secret="test1-secret-key",
                scope="openid profile phone offline_access".split(),
            ),
            headers={"Content-Type": "application/x-www-form-urlencoded"},
        )


def test_body_url_coded():
    pact = (
        Pact("Consumer", "Provider", file_write_mode="never")
        .given("everything is ideal")
        .upon_receiving("a request")
        .with_request(
            method="POST",
            path="/",
            headers={"Content-Type": "application/x-www-form-urlencoded"},
            body={
                "client_id": ["test1"],
                "secret": ["test1-secret-key"],
                "scope": ["openid", "profile", "phone", "offline_access"],
            },
        )
        .will_respond_with(200, body="some data")
    )
    with pact:
        requests.post(
            pact.uri,
            data=dict(
                client_id="test1",
                secret="test1-secret-key",
                scope="openid profile phone offline_access".split(),
            ),
            headers={"Content-Type": "application/x-www-form-urlencoded"},
        )



from queue import Queue
from unittest.mock import Mock

import pytest
from pactman.mock import mock_server


def queue(*a):
    q = Queue()
    for value in a:
        q.put(value)
    return q


@pytest.mark.parametrize(
    "results,exception",
    [
        (queue(dict(status="error", reason="spam")), mock_server.MockServer.Error),
        (queue(dict(status="failed", reason="spam")), AssertionError),
    ],
)
def test_correct_result_assertion(mocker, results, exception):
    mocker.patch("pactman.mock.mock_server.Process", autospec=True)
    s = mock_server.Server(Mock())
    s.results = results
    with pytest.raises(exception) as e:
        s.verify()
    assert "spam" in str(e.value)



import glob
import logging
import os
import warnings

import pytest
from _pytest.outcomes import Failed
from _pytest.reports import TestReport

from .broker_pact import BrokerPact, BrokerPacts, PactBrokerConfig
from .result import PytestResult, log


def pytest_addoption(parser):
    group = parser.getgroup("pact specific options (pactman)")
    group.addoption(
        "--pact-files", default=None, help="pact JSON files to verify (wildcards allowed)"
    )
    group.addoption("--pact-broker-url", default="", help="pact broker URL")
    group.addoption("--pact-broker-token", default="", help="pact broker bearer token")
    group.addoption(
        "--pact-provider-name", default=None, help="pact name of provider being verified"
    )
    group.addoption(
        "--pact-consumer-name",
        default=None,
        help="consumer name to limit pact verification to - "
        "DEPRECATED, use --pact-verify-consumer instead",
    )
    group.addoption(
        "--pact-verify-consumer", default=None, help="consumer name to limit pact verification to"
    )
    group.addoption(
        "--pact-verify-consumer-tag",
        metavar="TAG",
        action="append",
        help="limit broker pacts verified to those matching the tag. May be "
        "specified multiple times in which case pacts matching any of these "
        "tags will be verified.",
    )
    group.addoption(
        "--pact-publish-results",
        action="store_true",
        default=False,
        help="report pact verification results to pact broker",
    )
    group.addoption(
        "--pact-provider-version",
        default=None,
        help="provider version to use when reporting pact results to pact broker",
    )
    group.addoption(
        "--pact-allow-fail",
        default=False,
        action="store_true",
        help="do not fail the pytest run if any pacts fail verification",
    )


# Future options to be implemented. Listing them here so naming consistency can be a thing.
#    group.addoption("--pact-publish-pacts", action="store_true", default=False,
#                    help="publish pacts to pact broker")
#    group.addoption("--pact-consumer-version", default=None,
#                    help="consumer version to use when publishing pacts to the broker")
#    group.addoption("--pact-consumer-version-source", default=None,
#                    help="generate consumer version from source 'git-tag' or 'git-hash'")
#    group.addoption("--pact-consumer-version-tag", metavar='TAG', action="append",
#                    help="tag(s) that should be applied to the consumer version when pacts "
#                         "are uploaded to the broker; multiple tags may be supplied")


def get_broker_url(config):
    return config.getoption("pact_broker_url") or os.environ.get("PACT_BROKER_URL")


# add the pact broker URL to the pytest output if running verbose
def pytest_report_header(config):
    if config.getoption("verbose") > 0:
        location = get_broker_url(config) or config.getoption("pact_files")
        return [f"Loading pacts from {location}"]


def pytest_configure(config):
    logging.getLogger("pactman").handlers = []
    logging.basicConfig(format="%(message)s")
    verbosity = config.getoption("verbose")
    if verbosity > 0:
        log.setLevel(logging.DEBUG)


class PytestPactVerifier:
    def __init__(self, publish_results, provider_version, interaction, consumer):
        self.publish_results = publish_results
        self.provider_version = provider_version
        self.interaction = interaction
        self.consumer = consumer

    def verify(self, provider_url, provider_setup, extra_provider_headers={}):
        try:
            self.interaction.verify_with_callable_setup(provider_url, provider_setup, extra_provider_headers)
        except (Failed, AssertionError) as e:
            raise Failed(str(e)) from None

    def finish(self):
        if self.consumer and self.publish_results and self.provider_version:
            self.consumer.publish_result(self.provider_version)


def flatten_pacts(pacts):
    for consumer in pacts:
        last = consumer.interactions[-1]
        for interaction in consumer.interactions:
            if interaction is last:
                yield (interaction, consumer)
            else:
                yield (interaction, None)


def get_pact_files(file_location):
    if not file_location:
        return []
    for filename in glob.glob(file_location):
        yield BrokerPact.load_file(filename, result_factory=PytestResult)


def test_id(identifier):
    interaction, _ = identifier
    return str(interaction)


def pytest_generate_tests(metafunc):
    if "pact_verifier" in metafunc.fixturenames:
        broker_url = get_broker_url(metafunc.config)
        if not broker_url:
            pact_files = get_pact_files(metafunc.config.getoption("pact_files"))
            if not pact_files:
                raise ValueError("need a --pact-broker-url or --pact-files option")
            metafunc.parametrize(
                "pact_verifier", flatten_pacts(pact_files), ids=test_id, indirect=True
            )
        else:
            provider_name = metafunc.config.getoption("pact_provider_name")
            if not provider_name:
                raise ValueError("--pact-broker-url requires the --pact-provider-name option")
            broker = PactBrokerConfig(
                broker_url,
                metafunc.config.getoption("pact_broker_token"),
                metafunc.config.getoption("pact_verify_consumer_tag", []),
            )
            broker_pacts = BrokerPacts(
                provider_name, pact_broker=broker, result_factory=PytestResult
            )
            pacts = broker_pacts.consumers()
            filter_consumer_name = metafunc.config.getoption("pact_verify_consumer")
            if not filter_consumer_name:
                filter_consumer_name = metafunc.config.getoption("pact_consumer_name")
                if filter_consumer_name:
                    warnings.warn(
                        "The --pact-consumer-name command-line option is deprecated "
                        "and will be removed in the 3.0.0 release.",
                        DeprecationWarning,
                    )
            if filter_consumer_name:
                pacts = [pact for pact in pacts if pact.consumer == filter_consumer_name]
            metafunc.parametrize("pact_verifier", flatten_pacts(pacts), ids=test_id, indirect=True)


class PactTestReport(TestReport):
    """Custom TestReport that allows us to attach an interaction to the result, and
    then display the interaction's verification result ouput as well as the traceback
    of the failure.
    """

    @classmethod
    def from_item_and_call(cls, item, call, interaction):
        report = super().from_item_and_call(item, call)
        report.pact_interaction = interaction
        # the toterminal() call can't reasonably get at this config, so we store it here
        report.verbosity = item.config.option.verbose
        return report

    def toterminal(self, out):
        out.line("Pact failure details:", bold=True)
        for text, kw in self.pact_interaction.result.results_for_terminal():
            out.line(text, **kw)
        if self.verbosity > 0:
            out.line("Traceback:", bold=True)
            return super().toterminal(out)
        else:
            out.line("Traceback not shown, use pytest -v to show it")


def pytest_runtest_makereport(item, call):
    if call.when != "call" or "pact_verifier" not in getattr(item, "fixturenames", []):
        return
    # use our custom TestReport subclass if we're reporting on a pact verification call
    interaction = item.funcargs["pact_verifier"].interaction
    report = PactTestReport.from_item_and_call(item, call, interaction)
    if report.failed and item.config.getoption("pact_allow_fail"):
        # convert the fail into an "expected" fail, which allows the run to pass
        report.wasxfail = True
        report.outcome = "passed"
    return report


def pytest_report_teststatus(report, config):
    if not hasattr(report, "pact_interaction"):
        return
    if hasattr(report, "wasxfail"):
        # wasxfail usually displays an "X" but since it's not *expected* to fail an "f" is a little clearer
        return "ignore fail", "f", "IGNORE_FAIL"


@pytest.fixture()
def pact_verifier(pytestconfig, request):
    interaction, consumer = request.param
    p = PytestPactVerifier(
        pytestconfig.getoption("pact_publish_results"),
        pytestconfig.getoption("pact_provider_version"),
        interaction,
        consumer,
    )
    yield p
    p.finish()



from pactman import Term


def test_regex():
    assert Term("[a-zA-Z]", "abcXYZ").ruby_protocol() == {
        "json_class": "Pact::Term",
        "data": {
            "matcher": {"json_class": "Regexp", "s": "[a-zA-Z]", "o": 0},
            "generate": "abcXYZ",
        },
    }



import pytest
from pactman import Consumer, Provider


@pytest.mark.parametrize("version", ["2.0.0", "3.0.0"])
def test_null(version):
    pact = Consumer("consumer").has_pact_with(Provider("provider"), version=version)
    pact.given(None).upon_receiving("a request").with_request("GET", "/path").will_respond_with(
        200, body="ok"
    )

    result = pact.construct_pact(pact._interactions[0])
    assert result == {
        "consumer": {"name": "consumer"},
        "provider": {"name": "provider"},
        "interactions": [
            {
                "description": "a request",
                "request": dict(method="GET", path="/path"),
                "response": dict(status=200, body="ok"),
            }
        ],
        "metadata": dict(pactSpecification=dict(version=version)),
    }


def test_v2():
    pact = Consumer("consumer").has_pact_with(Provider("provider"), version="2.0.0")
    pact.given("the condition exists").upon_receiving("a request").with_request(
        "GET", "/path"
    ).will_respond_with(200, body="ok")

    result = pact.construct_pact(pact._interactions[0])
    assert result == {
        "consumer": {"name": "consumer"},
        "provider": {"name": "provider"},
        "interactions": [
            {
                "description": "a request",
                "providerState": "the condition exists",
                "request": dict(method="GET", path="/path"),
                "response": dict(status=200, body="ok"),
            }
        ],
        "metadata": dict(pactSpecification=dict(version="2.0.0")),
    }


def test_v3():
    pact = Consumer("consumer").has_pact_with(Provider("provider"), version="3.0.0")
    pact.given(
        [
            dict(name="the condition exists", params={}),
            dict(name="the user exists", params=dict(username="alex")),
        ]
    ).upon_receiving("a request").with_request("GET", "/path").will_respond_with(200, body="ok")

    result = pact.construct_pact(pact._interactions[0])
    assert result == {
        "consumer": {"name": "consumer"},
        "provider": {"name": "provider"},
        "interactions": [
            {
                "description": "a request",
                "providerStates": [
                    {"name": "the condition exists", "params": {}},
                    {"name": "the user exists", "params": {"username": "alex"}},
                ],
                "request": dict(method="GET", path="/path"),
                "response": dict(status=200, body="ok"),
            }
        ],
        "metadata": dict(pactSpecification=dict(version="3.0.0")),
    }


def test_v3_and_given():
    pact = (
        Consumer("consumer")
        .has_pact_with(Provider("provider"), version="3.0.0")
        .given("the condition exists")
        .and_given("the user exists", username="alex")
        .upon_receiving("a request")
        .with_request("GET", "/path")
        .will_respond_with(200, body="ok")
    )

    result = pact.construct_pact(pact._interactions[0])
    assert result == {
        "consumer": {"name": "consumer"},
        "provider": {"name": "provider"},
        "interactions": [
            {
                "description": "a request",
                "providerStates": [
                    {"name": "the condition exists", "params": {}},
                    {"name": "the user exists", "params": {"username": "alex"}},
                ],
                "request": dict(method="GET", path="/path"),
                "response": dict(status=200, body="ok"),
            }
        ],
        "metadata": dict(pactSpecification=dict(version="3.0.0")),
    }



import pytest
import requests
from pactman import Consumer, Equals, Like, Provider


@pytest.mark.parametrize("object", [None, list(), dict(), 1, 1.0, "string"])
def test_valid_types(object):
    Equals(object)


@pytest.mark.parametrize("object", [set(), b"bytes"])
def test_invalid_types(object):
    with pytest.raises(AssertionError) as e:
        Equals(object)

    assert "matcher must be one of " in str(e.value)


def test_basic_type():
    assert Equals(123).generate_matching_rule_v3() == {"matchers": [{"match": "equality"}]}


def test_v2_not_allowed():
    with pytest.raises(Equals.NotAllowed):
        Consumer("C").has_pact_with(Provider("P"), version="2.0.0").given("g").upon_receiving(
            "r"
        ).with_request("post", "/foo", body=Equals("bee")).will_respond_with(200)


def test_mock_usage_pass_validation():
    pact = (
        Consumer("C")
        .has_pact_with(Provider("P"), version="3.0.0")
        .given("g")
        .upon_receiving("r")
        .with_request("post", "/foo", body=Like({"a": "spam", "b": Equals("bee")}))
        .will_respond_with(200)
    )

    with pact:
        requests.post(pact.uri + "/foo", json={"a": "ham", "b": "bee"})


def test_mock_usage_fail_validation():
    pact = (
        Consumer("C")
        .has_pact_with(Provider("P"), version="3.0.0")
        .given("g")
        .upon_receiving("r")
        .with_request("post", "/foo", body=Like({"a": "spam", "b": Equals("bee")}))
        .will_respond_with(200)
    )

    with pytest.raises(AssertionError), pact:
        requests.post(pact.uri + "/foo", json={"a": "ham", "b": "wasp"})



import os

from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))

about = {}
with open(os.path.join(here, "pactman", "__version__.py")) as f:
    exec(f.read(), about)


def read(filename):
    with open(os.path.join(here, filename), "rb") as f:
        return f.read().decode("utf-8")


setup(
    name="pactman",
    version=about["__version__"],
    description=(
        "Tools for creating and verifying consumer driven contracts" " using the Pact framework."
    ),
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    author="ReeceTech",
    author_email="richard.jones@reece.com.au",
    url="https://github.com/reecetech/pactman",
    entry_points={
        "pytest11": ["pactman-verifier=pactman.verifier.pytest_plugin"],
        "console_scripts": ["pactman-verifier=pactman.verifier.command_line:main"],
    },
    install_requires=["pytest", "requests", "semver", "colorama", "restnavigator"],
    packages=find_packages(),
    license="MIT, Copyright (c) 2018 ReeceTech",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Topic :: Software Development :: Testing",
        "Topic :: Software Development :: Testing :: Mocking",
        "Topic :: Software Development :: Testing :: Acceptance",
    ],
)



import json
import os
import tempfile
from unittest.mock import Mock, mock_open, patch

import pactman.mock.pact
import pytest
import requests
import semver
from pactman.mock.consumer import Consumer
from pactman.mock.pact import Pact
from pactman.mock.pact_request_handler import (PactInteractionMismatch,
                                               PactRequestHandler,
                                               PactVersionConflict)
from pactman.mock.provider import Provider


@pytest.fixture
def mock_pact(monkeypatch):
    def f(file_write_mode=None, version="2.0.0"):
        monkeypatch.setattr(Pact, "allocate_port", Mock())
        monkeypatch.setattr(os, "remove", Mock())
        monkeypatch.setattr(os.path, "exists", Mock(return_value=True))
        log_dir = "/tmp/a"
        pact_dir = "/tmp/pact"
        return Pact(
            Consumer("CONSUMER"),
            Provider("PROVIDER"),
            log_dir=log_dir,
            pact_dir=pact_dir,
            version=version,
            file_write_mode=file_write_mode,
        )

    return f


@pytest.mark.parametrize("file_write_mode", [None, "overwrite"])
def test_pact_init(monkeypatch, file_write_mode, mock_pact):
    monkeypatch.setattr(pactman.mock.pact, "ensure_pact_dir", Mock(return_value=True))
    mock_pact = mock_pact(file_write_mode)
    filename = mock_pact.pact_json_filename

    assert mock_pact.consumer.name == "CONSUMER"
    assert mock_pact.provider.name == "PROVIDER"
    assert mock_pact.log_dir == "/tmp/a"
    assert mock_pact.version == "2.0.0"
    assert mock_pact.semver == semver.VersionInfo.parse("2.0.0")
    mock_pact.allocate_port.assert_called_once_with()
    assert mock_pact.BASE_PORT_NUMBER >= 8150
    pactman.mock.pact.ensure_pact_dir.assert_called_once_with("/tmp/pact")
    if file_write_mode == "overwrite":
        os.path.exists.assert_called_once_with(filename)
        os.remove.assert_called_once_with(filename)
    else:
        os.path.exists.assert_not_called()
        os.remove.assert_not_called()


def test_config_pact_filename(mock_pact):
    mock_pact = mock_pact()
    assert mock_pact.pact_json_filename == os.path.join(
        mock_pact.pact_dir, "CONSUMER-PROVIDER-pact.json"
    )


def test_ensure_pact_dir_when_exists(monkeypatch):
    monkeypatch.setattr(os.path, "exists", Mock(side_effect=[True]))
    monkeypatch.setattr(os, "mkdir", Mock())
    pactman.mock.pact.ensure_pact_dir("/tmp/pacts")
    os.mkdir.assert_not_called()


def test_ensure_pact_dir_when_parent_exists(monkeypatch):
    monkeypatch.setattr(os.path, "exists", Mock(side_effect=[False, True]))
    monkeypatch.setattr(os, "mkdir", Mock())
    pactman.mock.pact.ensure_pact_dir("/tmp/pacts")
    os.mkdir.assert_called_once_with("/tmp/pacts")


def generate_pact(version):
    return {
        "consumer": {"name": "CONSUMER"},
        "provider": {"name": "PROVIDER"},
        "interactions": [dict(description="spam")],
        "metadata": {"pactSpecification": {"version": version}},
    }


@pytest.mark.parametrize("version", ["2.0.0", "3.0.0"])
@patch("builtins.open", new_callable=mock_open, read_data="data")
def test_pact_request_handler_write_pact(mock_open, monkeypatch, mock_pact, version):
    monkeypatch.setattr(pactman.mock.pact, "ensure_pact_dir", Mock(return_value=True))
    mock_pact = mock_pact(version=version)
    mock_pact.semver = semver.VersionInfo.parse(version)
    my_pact = PactRequestHandler(mock_pact)
    os.path.exists.return_value = False
    with patch("json.dump", Mock()) as json_mock:
        my_pact.write_pact(dict(description="spam"))
        mock_open.assert_called_once_with(mock_pact.pact_json_filename, "w")
        json_mock.assert_called_once_with(generate_pact(version), mock_open(), indent=2)


@patch("builtins.open", new_callable=mock_open, read_data="data")
def test_versions_are_consistent(mock_open, monkeypatch, mock_pact):
    monkeypatch.setattr(pactman.mock.pact, "ensure_pact_dir", Mock(return_value=True))
    monkeypatch.setattr(json, "dump", Mock())
    monkeypatch.setattr(json, "load", lambda f: generate_pact("2.0.0"))

    # write the v2 pact
    pact = mock_pact()
    pact.semver = semver.VersionInfo.parse(pact.version)
    hdlr = PactRequestHandler(pact)
    hdlr.write_pact(dict(description="spam"))

    # try to add the v3 pact
    pact = mock_pact(version="3.0.0")
    pact.semver = semver.VersionInfo.parse(pact.version)
    hdlr = PactRequestHandler(pact)
    with pytest.raises(PactVersionConflict):
        hdlr.write_pact(dict(description="spam"))


def test_pacts_written():
    with tempfile.TemporaryDirectory() as d:
        pact = Consumer("C").has_pact_with(Provider("P"), pact_dir=d)
        with pact.given("g").upon_receiving("r").with_request("get", "/foo").will_respond_with(200):
            requests.get(pact.uri + "/foo")

        # force a failure
        with pytest.raises(AssertionError):
            with pact.given("g").upon_receiving("r2").with_request("get", "/bar").will_respond_with(
                200
            ):
                requests.get(pact.uri + "/foo")

        # make sure mocking still works
        with pact.given("g").upon_receiving("r2").with_request("get", "/bar").will_respond_with(
            200
        ):
            requests.get(pact.uri + "/bar")

        # ensure two pacts written
        with open(pact.pact_json_filename) as f:
            content = json.load(f)
            assert len(content["interactions"]) == 2


def test_detect_mismatch_request_manual_mode():
    with tempfile.TemporaryDirectory() as d:
        pact = (
            Consumer("C")
            .has_pact_with(Provider("P"), pact_dir=d, file_write_mode="merge")
            .given("g")
            .upon_receiving("r")
            .with_request("get", "/foo")
            .will_respond_with(200)
        )
        with pact:
            requests.get(pact.uri + "/foo")

        # force a failure by specifying the same given/providerState but different request
        pact = (
            Consumer("C")
            .has_pact_with(Provider("P"), pact_dir=d, file_write_mode="merge")
            .given("g")
            .upon_receiving("r")
            .with_request("get", "/bar")
            .will_respond_with(200)
        )
        with pytest.raises(PactInteractionMismatch):
            with pact:
                requests.get(pact.uri + "/bar")


def test_detect_mismatch_request_retained_relationship():
    with tempfile.TemporaryDirectory() as d:
        pact = Consumer("C").has_pact_with(Provider("P"), pact_dir=d)
        with pact.given("g").upon_receiving("r").with_request("get", "/foo").will_respond_with(200):
            requests.get(pact.uri + "/foo")

        # force a failure by specifying the same given/providerState but different request
        with pytest.raises(PactInteractionMismatch):
            with pact.given("g").upon_receiving("r").with_request("get", "/bar").will_respond_with(
                200
            ):
                requests.get(pact.uri + "/bar")



"""Python methods for interactive with a Pact Mock Service."""
from .mock.consumer import Consumer
from .mock.matchers import EachLike, Equals, Includes, Like, SomethingLike, Term
from .mock.pact import Pact
from .mock.provider import Provider

__all__ = (
    "Consumer",
    "EachLike",
    "Equals",
    "Includes",
    "Like",
    "Pact",
    "Provider",
    "SomethingLike",
    "Term",
)



from pactman import Consumer, Provider


def test_full_payload_v2():
    pact = Consumer("consumer").has_pact_with(Provider("provider"), version="2.0.0")
    (
        pact.given("UserA exists and is not an administrator")
        .upon_receiving("a request for UserA")
        .with_request(
            "get", "/users/UserA", headers={"Accept": "application/json"}, query="term=test"
        )
        .will_respond_with(
            200, body={"username": "UserA"}, headers={"Content-Type": "application/json"}
        )
    )
    result = pact.construct_pact(pact._interactions[0])
    assert result == {
        "consumer": {"name": "consumer"},
        "provider": {"name": "provider"},
        "interactions": [
            {
                "description": "a request for UserA",
                "providerState": "UserA exists and is not an administrator",
                "request": dict(
                    method="get",
                    path="/users/UserA",
                    headers={"Accept": "application/json"},
                    query="term=test",
                ),
                "response": dict(
                    status=200,
                    body={"username": "UserA"},
                    headers={"Content-Type": "application/json"},
                ),
            }
        ],
        "metadata": dict(pactSpecification=dict(version="2.0.0")),
    }


def test_full_payload_v3():
    pact = Consumer("consumer").has_pact_with(Provider("provider"), version="3.0.0")
    (
        pact.given(
            [{"name": "User exists and is not an administrator", "params": {"username": "UserA"}}]
        )
        .upon_receiving("a request for UserA")
        .with_request(
            "get", "/users/UserA", headers={"Accept": "application/json"}, query=dict(term=["test"])
        )
        .will_respond_with(
            200, body={"username": "UserA"}, headers={"Content-Type": "application/json"}
        )
    )
    result = pact.construct_pact(pact._interactions[0])
    assert result == {
        "consumer": {"name": "consumer"},
        "provider": {"name": "provider"},
        "interactions": [
            {
                "description": "a request for UserA",
                "providerStates": [
                    {
                        "name": "User exists and is not an administrator",
                        "params": {"username": "UserA"},
                    }
                ],
                "request": dict(
                    method="get",
                    path="/users/UserA",
                    headers={"Accept": "application/json"},
                    query=dict(term=["test"]),
                ),
                "response": dict(
                    status=200,
                    body={"username": "UserA"},
                    headers={"Content-Type": "application/json"},
                ),
            }
        ],
        "metadata": dict(pactSpecification=dict(version="3.0.0")),
    }



from unittest.mock import Mock

from pactman.verifier import result


def test_logged_start_result(monkeypatch):
    monkeypatch.setattr(result, "log", Mock())
    r = result.LoggedResult()
    r.start(Mock())
    result.log.info.assert_called_once()


def test_logged_fail_result(monkeypatch):
    monkeypatch.setattr(result, "log", Mock())
    r = result.LoggedResult()
    r.fail("message")
    result.log.warning.assert_called_once_with(" message")


def test_logged_warning(monkeypatch):
    monkeypatch.setattr(result, "log", Mock())
    r = result.LoggedResult()
    r.warn("message")
    result.log.warning.assert_called_once_with(" message")


def test_logged_fail_result_path(monkeypatch):
    monkeypatch.setattr(result, "log", Mock())
    r = result.LoggedResult()
    r.fail("message", ["a", 0])
    result.log.warning.assert_called_once_with(" message")


def test_CaptureResult_for_passing_verification(capsys):
    r = result.CaptureResult()
    r.start(Mock())
    r.end()
    captured = capsys.readouterr()
    assert "PASSED" in captured.out


def test_CaptureResult_for_failing_verification(capsys):
    r = result.CaptureResult()
    r.start(Mock())
    r.fail("message1")
    r.end()
    captured = capsys.readouterr()
    assert "FAILED" in captured.out
    assert "message1" in captured.out


def test_CaptureResult_for_fail_with_path(capsys):
    r = result.CaptureResult()
    r.start(Mock())
    r.fail("message1", path=["x", "y"])
    r.end()
    captured = capsys.readouterr()
    assert "message1 at x.y" in captured.out


def test_CaptureResult_for_passing_verification_with_warning(capsys):
    r = result.CaptureResult()
    r.start(Mock())
    r.warn("message1")
    r.end()
    captured = capsys.readouterr()
    assert "PASSED" in captured.out
    assert "message1" in captured.out



import argparse
import logging
from functools import partial

from colorama import init

from ..__version__ import __version__
from .broker_pact import BrokerPact, BrokerPacts, PactBrokerConfig
from .result import CaptureResult

# TODO: add these options, which exist in the ruby command line?
"""
@click.option(
    'timeout', '-t', '--timeout',
    default=30,
    help='The duration in seconds we should wait to confirm verification'
         ' process was successful. Defaults to 30.',
    type=int)
"""

parser = argparse.ArgumentParser(description="Verify pact contracts")

parser.add_argument(
    "provider_name", metavar="PROVIDER_NAME", help="the name of the provider being verified"
)

parser.add_argument("provider_url", metavar="PROVIDER_URL", help="the URL of the provider service")

parser.add_argument(
    "provider_setup_url", metavar="PROVIDER_SETUP_URL", help="the URL to provider state setup"
)

parser.add_argument(
    "-b",
    "--broker-url",
    default=None,
    help="the URL of the pact broker which may include basic auth; "
    "may also be provided in PACT_BROKER_URL env var",
)

parser.add_argument(
    "--broker-token",
    default=None,
    help="pact broker bearer token; may also be provided in PACT_BROKER_TOKEN env var",
)

parser.add_argument("-l", "--local-pact-file", default=None, help="path to a local pact file")

parser.add_argument("-c", "--consumer", default=None, help="the name of the consumer to test")

parser.add_argument(
    "--consumer-version-tag",
    metavar="TAG",
    action="append",
    help="limit broker pacts tested to those matching the tag. May be specified "
    "multiple times in which case pacts matching any of these tags will be "
    "verified.",
)

parser.add_argument(
    "--custom-provider-header",
    metavar="PROVIDER_EXTRA_HEADER",
    action="append",
    help="Header to add to provider state set up and pact verification requests. "
    'eg "Authorization: Basic cGFjdDpwYWN0". May be specified multiple times.',
)

parser.add_argument(
    "-r",
    "--publish-verification-results",
    default=False,
    action="store_true",
    help="send verification results to the pact broker",
)

parser.add_argument(
    "-a",
    "--provider-app-version",
    default=None,
    help="provider application version, required for results publication (same as -p)",
)

parser.add_argument(
    "-p",
    "--provider-version",
    default=None,
    help="provider application version, required for results publication (same as -a)",
)

parser.add_argument(
    "-v",
    "--verbose",
    default=False,
    action="store_true",
    help="output more information about the verification",
)

parser.add_argument(
    "-q",
    "--quiet",
    default=False,
    action="store_true",
    help="output less information about the verification",
)

parser.add_argument(
    "-V", "--version", default=False, action="version", version=f"%(prog)s {__version__}"
)


def main():
    init(autoreset=True)
    args = parser.parse_args()
    provider_version = args.provider_version or args.provider_app_version
    custom_headers = get_custom_headers(args)
    if args.publish_verification_results and not provider_version:
        print("Provider version is required to publish results to the broker")
        return False
    pacts = get_pacts(args)
    success = True
    for pact in pacts:
        if args.consumer and pact.consumer != args.consumer:
            continue
        for interaction in pact.interactions:
            interaction.verify(
                args.provider_url, args.provider_setup_url, extra_provider_headers=custom_headers
            )
            success = interaction.result.success and success
        if args.publish_verification_results:
            pact.publish_result(provider_version)
        else:
            print()
    return int(not success)


def get_pacts(args):
    result_log_level = get_log_level(args)
    result_factory = partial(CaptureResult, level=result_log_level)
    if args.local_pact_file:
        pacts = [BrokerPact.load_file(args.local_pact_file, result_factory)]
    else:
        broker_config = PactBrokerConfig(
            args.broker_url, args.broker_token, args.consumer_version_tag
        )
        pacts = BrokerPacts(args.provider_name, broker_config, result_factory).consumers()
    return pacts


def get_log_level(args):
    if args.quiet:
        result_log_level = logging.WARNING
    elif args.verbose:
        result_log_level = logging.DEBUG
    else:
        result_log_level = logging.INFO
    return result_log_level


def get_custom_headers(args):
    custom_headers = {}
    if args.custom_provider_header:
        for header in args.custom_provider_header:
            k, v = header.split(":")
            custom_headers[k] = v.strip()
    return custom_headers


if __name__ == "__main__":
    import sys

    sys.exit(main())
